// CircleObj.cpp : implementation file
//

#include "stdafx.h"
#include "math.h"
#include "IA369J.h"
#include "CircleObj.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


BOOL CRectEx::PtInEllipse(POINT pt)
{
	if(!PtInRect(pt)) return FALSE;
	double cx = left + right;
	double cy = top + bottom;
	double w = Width();
	double h = Height();

	double t1 = (2*pt.x - cx) / w;
	double t2 = (2*pt.y - cy) / h;

	return t1*t1 + t2*t2 <=1;
}

/////////////////////////////////////////////////////////////////////////////
// CCircleObj

IMPLEMENT_SERIAL(CCircleObj,  CGraphObj,  VERSIONABLE_SCHEMA | 1)

CCircleObj::CCircleObj() : m_center(40, 40), m_radius(30)
{
	m_dragstat = NODRAG;
	m_cor = CYAN;
}

CCircleObj::~CCircleObj()
{
}

void CCircleObj::Serialize(CArchive& ar)
{
	CGraphObj::Serialize(ar);
	if(ar.IsStoring())
	{
		ar << m_center;
		ar << m_radius;
	}
	else
	{
		ar >> m_center;
		ar >> m_radius;
	}
}

void CCircleObj::GetDrawInfo(CRect& rect, CPoint* phl, CPoint* pht, CPoint* phr, CPoint* phb)
{
	CPoint pt = m_center + m_ptRef;
	rect = CRect(pt.x, pt.y, pt.x, pt.y);
	int raio = m_radius;
	switch(m_dragstat)
	{
	case DRAGHANDLE:
		{
			CSize sz(pt - m_ptDragNow);
			raio = (int)sqrt(sz.cx*sz.cx + sz.cy*sz.cy);
			rect.InflateRect(raio, raio);
			if(phl) *phl = m_ptDragNow;
			if(pht) *pht = m_ptDragNow;
			if(phr) *phr = m_ptDragNow;
			if(phb) *phb = m_ptDragNow;
		}
		break;

	case DRAGALL:
		rect.OffsetRect(m_ptDragNow - m_ptDragOrg);

	case NODRAG:
		{
			rect.InflateRect(raio, raio);
			CPoint cp = rect.CenterPoint();
			if(phl) *phl = cp + CSize(-raio, 0);
			if(pht) *pht = cp + CSize(0, -raio);
			if(phr) *phr = cp + CSize(raio, 0);
			if(phb) *phb = cp + CSize(0, raio);
		}

		break;
	}
}

DWORD CCircleObj::Test_point(POINT& pt)
{
	CPoint ptt;
	CPoint ptl;
	CPoint ptb;
	CPoint ptr;
	CRectEx rect;

	GetDrawInfo(rect, &ptl, &ptt, &ptr, &ptb);

	DWORD dwRet = 0;
	
	TestHandle(pt, ptl, dwRet);
	TestHandle(pt, ptt, dwRet);
	TestHandle(pt, ptr, dwRet);
	TestHandle(pt, ptb, dwRet);

	if(rect.PtInEllipse(pt))
	{
			dwRet |= PTTEST_INSIDE;
	}

	return dwRet;
}

void CCircleObj::Bound_rectangle(CRect& rect)
{
	GetDrawInfo(rect);
	rect.InflateRect(HANDLE_SIZE+1, HANDLE_SIZE+1);
}

void CCircleObj::Draw(CDC* pDC, BOOL bSelected)
{
	CRect rect;
	CPoint ptt, ptl, ptr, ptb;
	GetDrawInfo(rect, &ptl, &ptt, &ptr, &ptb);

	CPen  pen(PS_NULL, 0, RGB(0,0,0));
	CBrush brush(m_cor);
	CPen* pOldPen = (CPen*)pDC->SelectObject(&pen);
	CBrush* pOldBrush = (CBrush*)pDC->SelectObject(&brush);
	pDC->Ellipse(&rect);

	pDC->SelectObject(pOldPen);
	pDC->SelectObject(pOldBrush);
	
	if(bSelected)
	{
		DrawHandle(pDC, ptl);
		DrawHandle(pDC, ptt);
		DrawHandle(pDC, ptb);
		DrawHandle(pDC, ptr);
	}
}

BOOL CCircleObj::Grab_point(POINT& pt, BOOL bSelected)
{
	ASSERT(m_dragstat == NODRAG);
	CRectEx rect;
	CPoint ptt, ptl, ptr, ptb;
	GetDrawInfo(rect, &ptl, &ptt, &ptr, &ptb);
	DWORD dw = 0;
	if(bSelected)
	{
		TestHandle(pt, ptl, dw);
		TestHandle(pt, ptt, dw);
		TestHandle(pt, ptr, dw);
		TestHandle(pt, ptb, dw);
		if(dw)
		{
			m_dragstat = DRAGHANDLE;
			m_ptDragNow = m_ptDragOrg = pt;
			return TRUE;
		}
	}
	
	if(rect.PtInEllipse(pt))
	{
		m_dragstat = DRAGALL;
		m_ptDragNow = m_ptDragOrg = pt;
		return TRUE;
	}

	return FALSE;
}

BOOL CCircleObj::Release_point(POINT& pt, BOOL Success, RECT* pRect)
{
	if(m_dragstat == NODRAG)
	{
		return FALSE;
	}

	CSize szoff(CPoint(pt) - m_ptDragOrg);
	if(Success)
	{
		Move_point(pt, pRect);
		CRect rect;
		GetDrawInfo(rect);
		switch(m_dragstat)
		{
		case DRAGHANDLE:
			m_radius = rect.Width() / 2;
			break;

		case DRAGALL:
			m_ptRef += szoff;
			break;
		}
		Normalize();
	}
	else
	{
		Move_point(m_ptDragOrg, pRect);
	}
	m_dragstat = NODRAG;
	return TRUE;
}

BOOL CCircleObj::Move_point(POINT& pt, RECT* pRect)
{
	if(m_dragstat == NODRAG)
	{
		return FALSE;
	}

	CRect rect1, rect2;
	Bound_rectangle(rect1);
	m_ptDragNow = pt;
	if(pRect)
	{
		Bound_rectangle(rect2);
		*pRect = rect1 | rect2;
	}
	return TRUE;
}

BOOL CCircleObj::Show_properties()
{
	return FALSE;
}

void CCircleObj::Normalize()
{
	CPoint ptc = m_center + m_ptRef;
	m_ptRef.x = ptc.x - m_radius;
	m_ptRef.y = ptc.y - m_radius;
	m_center = CPoint(m_radius, m_radius);
}

/////////////////////////////////////////////////////////////////////////////
// CCircleObj message handlers
