// IA369JDoc.cpp : implementation of the CIA369JDoc class
//

#include "stdafx.h"
#include "IA369J.h"

#include "CircleObj.h"
#include "LineObj.h"
#include "RectObj.h"
#include "TriangObj.h"
#include "BitmapObj.h"

#include "IA369JDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CIA369JDoc

IMPLEMENT_DYNCREATE(CIA369JDoc, CDocument)

BEGIN_MESSAGE_MAP(CIA369JDoc, CDocument)
	//{{AFX_MSG_MAP(CIA369JDoc)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIA369JDoc construction/destruction

CIA369JDoc::CIA369JDoc()
{
	m_pDefCircle = NULL;
	m_pDefLine = NULL;
	m_pDefRect = NULL;
	m_pDefTriang = NULL;
	m_bkColor = BRANCO;
}

CIA369JDoc::~CIA369JDoc()
{
}

BOOL CIA369JDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	m_pDefCircle = new CCircleObj;
	m_pDefLine = new CLineObj;
	m_pDefRect = new CRectObj;
	m_pDefTriang = new CTriangObj;
	m_bkColor = BRANCO;

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CIA369JDoc serialization

void CIA369JDoc::Serialize(CArchive& ar)
{
	CDocument::Serialize(ar);
	if (ar.IsStoring())
	{
		ar << m_pDefCircle;
		ar << m_pDefLine;
		ar << m_pDefRect;
		ar << m_pDefTriang;
		ar << m_bkColor;
	}
	else
	{
		ar >> m_pDefCircle;
		ar >> m_pDefLine;
		ar >> m_pDefRect;
		ar >> m_pDefTriang;
		ar >> m_bkColor;
	}

	m_objList.Serialize(ar);
}

/////////////////////////////////////////////////////////////////////////////
// CIA369JDoc diagnostics

#ifdef _DEBUG
void CIA369JDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CIA369JDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CIA369JDoc commands

void CIA369JDoc::DeleteContents() 
{
	delete m_pDefCircle;
	delete m_pDefLine;
	delete m_pDefRect;
	delete m_pDefTriang;
	
	POSITION pos = m_objList.GetHeadPosition();
	while(pos)
	{
		delete m_objList.GetNext(pos);
	}
	m_objList.RemoveAll();

	CDocument::DeleteContents();
}


void CIA369JDoc::Draw(CDC* pDC, CGraphObj* pSelObj)
{
	// Usar um contexto em memria para evitar flicker
	CDC memDC;
	memDC.CreateCompatibleDC(pDC);
	CRect totalRect;
	pDC->GetWindow()->GetClientRect(&totalRect);
	CBitmap bm;
	bm.CreateCompatibleBitmap(pDC, totalRect.Width(), totalRect.Height());
	CBitmap* pOldBitmap = (CBitmap*)memDC.SelectObject(&bm);

	memDC.SetViewportOrg(pDC->GetViewportOrg());
	memDC.DPtoLP(&totalRect);

	// apagar o fundo com a cor escolhida
	CBrush bc(m_bkColor);
	memDC.FillRect(&totalRect, &bc);


	BOOL bValidSel = FALSE;
	POSITION pos = m_objList.GetTailPosition();
	while(pos)
	{
		CGraphObj* pObj = m_objList.GetPrev(pos);
		if(pObj != pSelObj && !pObj->IsDeleted())
		{
			pObj->Draw(&memDC, FALSE);
		}
		else
		{
			bValidSel = TRUE;
		}
	}
	if(pSelObj && bValidSel)
	{
		pSelObj->Draw(&memDC, TRUE);
	}

	pDC->BitBlt(totalRect.left, totalRect.top, totalRect.Width(), totalRect.Height(),
		&memDC, totalRect.left, totalRect.top, SRCCOPY);

	memDC.SelectObject(pOldBitmap);
}

CGraphObj* CIA369JDoc::AddNewLine(POINT& ptRef)
{
	CGraphObj* pObj = m_pDefLine->Clone(&ptRef);
	m_objList.AddHead(pObj);
	return pObj;
}

CGraphObj* CIA369JDoc::AddNewCircle(POINT& ptRef)
{
	CGraphObj* pObj = m_pDefCircle->Clone(&ptRef);
	m_objList.AddHead(pObj);
	return pObj;
}

CGraphObj* CIA369JDoc::AddNewTriangle(POINT& ptRef)
{
	CGraphObj* pObj = m_pDefTriang->Clone(&ptRef);
	m_objList.AddHead(pObj);
	return pObj;
}

CGraphObj* CIA369JDoc::AddNewRectangle(POINT& ptRef)
{
	CGraphObj* pObj = m_pDefRect->Clone(&ptRef);
	m_objList.AddHead(pObj);
	return pObj;
}

CGraphObj* CIA369JDoc::AddNewBitmap(POINT& ptRef)
{
	CBitmapObj* pObj = new CBitmapObj;
	pObj->LoadDefBitmap();
	m_objList.AddHead(pObj);
	return pObj;
}

CGraphObj* CIA369JDoc::FromPoint(POINT& pt)
{
	POSITION pos = m_objList.GetHeadPosition();
	while(pos)
	{
		CGraphObj* pObj = m_objList.GetNext(pos);
		if(!pObj->IsDeleted())
		{
			DWORD dw = pObj->Test_point(pt);
			if(dw & PTTEST_INSIDE)
			{
				return pObj;
			}
		}
	}
	return NULL;
}

void CIA369JDoc::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(IsModified());
}

void CIA369JDoc::Delete(CGraphObj* pObj)
{
	POSITION pos = m_objList.Find(pObj);
	if(pos)
	{
		m_objList.RemoveAt(pos);
	}
	delete pObj;
}

void CIA369JDoc::Move_top(CGraphObj *pObj)
{
	POSITION pos = m_objList.Find(pObj);
	if(pos)
	{
		m_objList.RemoveAt(pos);
		m_objList.AddHead(pObj);
	}
}

void CIA369JDoc::Move_bottom(CGraphObj *pObj)
{
	POSITION pos = m_objList.Find(pObj);
	if(pos)
	{
		m_objList.RemoveAt(pos);
		m_objList.AddTail(pObj);
	}
}

BOOL CIA369JDoc::IsTop(CGraphObj* pObj)
{
	return pObj == m_objList.GetHead();
}

BOOL CIA369JDoc::IsBottom(CGraphObj* pObj)
{
	return pObj == m_objList.GetTail();
}
