// LineObj.cpp : implementation file
//

#include "stdafx.h"
#include "math.h"
#include "IA369J.h"
#include "LineObj.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLineObj

IMPLEMENT_SERIAL(CLineObj, CGraphObj,  VERSIONABLE_SCHEMA | 1)

CLineObj::CLineObj() : m_pt1(20, 20), m_pt2(120, 20)
{
	m_dragstat = NODRAG;
}

CLineObj::~CLineObj()
{
}

DWORD CLineObj::Test_point(POINT& pt)
{
	CPoint pt1 = m_pt1 + m_ptRef;
	CPoint pt2 = m_pt2 + m_ptRef;

	DWORD dwRet = 0;
	
	TestHandle(pt, pt1, dwRet);
	TestHandle(pt, pt2, dwRet);

	CRect rect;
	Bound_rectangle(rect);
	if(rect.PtInRect(pt))
	{
		double a = (double)(pt1.y - pt2.y);
		double b = (double)(pt2.x - pt1.x);
		double c = (double)(pt2.y*pt1.x - pt2.x*pt1.y);

		double rab = sqrt(a*a + b*b);

		double dist = (a*pt.x + b*pt.y + c) / rab;
		if(dist >= -HANDLE_SIZE && dist <= HANDLE_SIZE)
		{
			dwRet |= PTTEST_INSIDE;
		}
	}

	return dwRet;
}

void CLineObj::Bound_rectangle(CRect& rect)
{
	CPoint pt1;
	CPoint pt2;
	GetDrawPoints(pt1, pt2);
	rect = CRect(pt1, pt2);
	rect.NormalizeRect();
	rect.InflateRect(HANDLE_SIZE+1, HANDLE_SIZE+1);
}


void CLineObj::Draw(CDC* pDC, BOOL bSelected)
{
	CPen pen(m_style, 0, m_cor);
	CPen* pOldPen = (CPen*)pDC->SelectObject(&pen);
	CPoint pt1, pt2;
	GetDrawPoints(pt1, pt2);
	pDC->MoveTo(pt1);
	pDC->LineTo(pt2);
	pDC->SelectObject(pOldPen);

	if(bSelected)
	{
		DrawHandle(pDC, pt1);
		DrawHandle(pDC, pt2);
	}
}

void CLineObj::GetDrawPoints(CPoint& pt1, CPoint& pt2)
{
	//	enum {, , DRAGPT2, } m_dragstat;

	pt1 = m_ptRef + m_pt1;
	pt2 = m_ptRef + m_pt2;
	CSize off(m_ptDragNow - m_ptDragOrg);
	switch(m_dragstat)
	{
	case DRAGPT1:
		pt1 += off;
		break;

	case DRAGPT2:
		pt2 += off;
		break;

	case DRAGALL:
		pt1 += off;
		pt2 += off;
	}
}


BOOL CLineObj::Grab_point(POINT& pt, BOOL bSelected)
{
	ASSERT(m_dragstat == NODRAG);
	CPoint pt1(m_ptRef + m_pt1);
	CPoint pt2(m_ptRef + m_pt2);
	DWORD dw = 0;
	if(bSelected)
	{
		TestHandle(pt, pt1, dw);
		if(dw)
		{
			m_dragstat = DRAGPT1;
			m_ptDragNow = m_ptDragOrg = pt;
			return TRUE;
		}
		TestHandle(pt, pt2, dw);
		if(dw)
		{
			m_dragstat = DRAGPT2;
			m_ptDragNow = m_ptDragOrg = pt;
			return TRUE;
		}
	}
	
	dw = Test_point(pt);
	if(dw & PTTEST_INSIDE)
	{
		m_dragstat = DRAGALL;
		m_ptDragNow = m_ptDragOrg = pt;
		return TRUE;
	}

	return FALSE;
}

BOOL CLineObj::Release_point(POINT& pt, BOOL Success, RECT* pRect)
{
	if(m_dragstat == NODRAG)
	{
		return FALSE;
	}

	CSize szoff(CPoint(pt) - m_ptDragOrg);
	if(Success)
	{
		Move_point(pt, pRect);
		switch(m_dragstat)
		{
		case DRAGPT1:
			m_pt1 += szoff;
			break;

		case DRAGPT2:
			m_pt2 += szoff;
			break;

		case DRAGALL:
			m_ptRef += szoff;
			break;
		}
		Normalize();
	}
	else
	{
		Move_point(m_ptDragOrg, pRect);
	}
	m_dragstat = NODRAG;
	return TRUE;
}

void CLineObj::Normalize()
{
	CPoint pt1(m_ptRef + m_pt1);
	CPoint pt2(m_ptRef + m_pt2);

	CRect rect(pt1, pt2);
	rect.NormalizeRect();

	m_ptRef = rect.TopLeft();
	m_pt1 = pt1 - m_ptRef;
	m_pt2 = pt2 - m_ptRef;
}

BOOL CLineObj::Move_point(POINT& pt, RECT* pRect)
{
	// 	enum {NODRAG, DRAGPT1, DRAGPT2, DRAGALL} m_dragstat;

	if(m_dragstat == NODRAG)
	{
		return FALSE;
	}

	CRect rect1, rect2;
	Bound_rectangle(rect1);
	m_ptDragNow = pt;
	if(pRect)
	{
		Bound_rectangle(rect2);
		*pRect = rect1 | rect2;
	}
	return TRUE;
}

BOOL CLineObj::Show_properties()
{
	return FALSE;
}

void CLineObj::Serialize(CArchive& ar)
{
	CGraphObj::Serialize(ar);
	if(ar.IsStoring())
	{
		ar << m_pt1;
		ar << m_pt2;
		ar << m_style;
	}
	else
	{
		ar >> m_pt1;
		ar >> m_pt2;
		ar >> m_style;
	}
}
