// TriangObj.cpp : implementation file
//

#include "stdafx.h"
#include "IA369J.h"
#include "TriangObj.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTriangObj

IMPLEMENT_SERIAL(CTriangObj,CGraphObj,  VERSIONABLE_SCHEMA | 1)

CTriangObj::CTriangObj()
{
	m_dragstat = NODRAG;
	m_pt[0] = CPoint(20, 20);
	m_pt[1] = CPoint(20, 120);
	m_pt[2] = CPoint(120, 120);
	m_cor = VERMELHO;
}

CTriangObj::~CTriangObj()
{
}


void CTriangObj::GetDrawInfo(CPoint* ppt)
{
	int i;
	for(i=0; i<3; i++)
	{
		ppt[i] = m_pt[i] + m_ptRef;
	}

//	enum {NODRAG, DRAGPT0, DRAGPT1, DRAGPT2, DRAGALL} m_dragstat;
	if(m_dragstat >= DRAGPT0 && m_dragstat <= DRAGPT2)
	{
		ppt[m_dragstat - DRAGPT0] = m_ptDragNow;
	}

	else if(m_dragstat == DRAGALL)
	{
		for(i=0; i<3; i++)
		{
			ppt[i] += (m_ptDragNow - m_ptDragOrg);
		}
	}
}

DWORD CTriangObj::Test_point(POINT& pt)
{
	int i;
	CPoint ptt[3];
	DWORD dwRet = 0;

	GetDrawInfo(ptt);

	for(i=0; i<3; i++)
	{
		TestHandle(pt, ptt[i], dwRet);
	}

	if(	IsSameSide(ptt[1], ptt[2], ptt[0], pt) &&
		IsSameSide(ptt[0], ptt[2], ptt[1], pt) &&
		IsSameSide(ptt[0], ptt[1], ptt[2], pt))
	{
		dwRet |= PTTEST_INSIDE;
	}
	return dwRet;
}

void CTriangObj::Bound_rectangle(CRect& rect)
{
	CPoint pt[3];
	GetDrawInfo(pt);

	rect = CRect(pt[0], pt[0]);
	for(int i = 1; i<3; i++)
	{
		rect.top = min(rect.top, pt[i].y);
		rect.bottom = max(rect.bottom, pt[i].y);
		rect.left = min(rect.left, pt[i].x);
		rect.right = max(rect.right, pt[i].x);
	}
	rect.InflateRect(HANDLE_SIZE+1, HANDLE_SIZE+1);
}

void CTriangObj::Draw(CDC* pDC, BOOL bSelected)
{
	CPen  pen(PS_NULL, 0, RGB(0,0,0));
	CBrush brush(m_cor);
	CPen* pOldPen = (CPen*)pDC->SelectObject(&pen);
	CBrush* pOldBrush = (CBrush*)pDC->SelectObject(&brush);

	CPoint pt[3];
	GetDrawInfo(pt);

	pDC->Polygon(pt, 3);

	pDC->SelectObject(pOldPen);
	pDC->SelectObject(pOldBrush);
	
	if(bSelected)
	{
		for(int i=0; i<3; i++)
		{
			DrawHandle(pDC, pt[i]);
		}
	}
}

BOOL CTriangObj::Grab_point(POINT& pt, BOOL bSelected)
{
//	enum {NODRAG, DRAGPT0, DRAGPT1, DRAGPT2, DRAGALL} m_dragstat;
	ASSERT(m_dragstat == NODRAG);
	int i;
	CPoint ptt[3];
	GetDrawInfo(ptt);
	DWORD dw = 0;
	if(bSelected)
	{

		for(i=0; i<3; i++)
		{
			TestHandle(pt, ptt[i], dw);
			if(dw)
			{
				m_dragstat = (DRAGSTAT)(DRAGPT0 + i);
				m_ptDragNow = m_ptDragOrg = pt;
				return TRUE;
			}
		}
	}
	
	if(	IsSameSide(ptt[1], ptt[2], ptt[0], pt) &&
		IsSameSide(ptt[0], ptt[2], ptt[1], pt) &&
		IsSameSide(ptt[0], ptt[1], ptt[2], pt))
	{
		m_dragstat = DRAGALL;
		m_ptDragNow = m_ptDragOrg = pt;
		return TRUE;
	}

	return FALSE;
}

BOOL CTriangObj::Release_point(POINT& pt, BOOL Success, RECT* pRect)
{
	if(m_dragstat == NODRAG)
	{
		return FALSE;
	}

	CSize szoff(CPoint(pt) - m_ptDragOrg);
	if(Success)
	{
		Move_point(pt, pRect);
		CPoint ptt[3];
		GetDrawInfo(ptt);

//	enum {NODRAG, DRAGPT0, DRAGPT1, DRAGPT2, DRAGALL} m_dragstat;
		if(m_dragstat >= DRAGPT0 && m_dragstat <= DRAGPT2)
		{
			m_pt[m_dragstat - DRAGPT0] = CPoint(pt) - m_ptRef;
		}

		else if(m_dragstat == DRAGALL)
		{
			m_ptRef += szoff;
		}

		Normalize();
	}
	else
	{
		Move_point(m_ptDragOrg, pRect);
	}
	m_dragstat = NODRAG;
	return TRUE;
}

BOOL CTriangObj::Move_point(POINT& pt, RECT* pRect)
{
//	enum DRAGSTAT {NODRAG, DRAGPT0, DRAGPT1, DRAGPT2, DRAGALL} m_dragstat;

	if(m_dragstat == NODRAG)
	{
		return FALSE;
	}

	CRect rect1, rect2;
	Bound_rectangle(rect1);
	m_ptDragNow = pt;
	if(pRect)
	{
		Bound_rectangle(rect2);
		*pRect = rect1 | rect2;
	}
	return TRUE;
}

BOOL CTriangObj::Show_properties()
{
	return FALSE;
}

void CTriangObj::Normalize()
{
	int i;
	CPoint ptOrg = m_pt[0];
	for(i = 1; i < 3; i++)
	{
		ptOrg.x = min(ptOrg.x, m_pt[i].x);
		ptOrg.y = min(ptOrg.y, m_pt[i].y);
	}

	m_ptRef += ptOrg;

	for(i = 0; i < 3; i++)
	{
		m_pt[i] -= ptOrg;
	}
}

void CTriangObj::Serialize(CArchive& ar)
{
	CGraphObj::Serialize(ar);
	if(ar.IsStoring())
	{
		ar << m_pt[0];
		ar << m_pt[1];
		ar << m_pt[2];
	}
	else
	{
		ar >> m_pt[0];
		ar >> m_pt[1];
		ar >> m_pt[2];
	}
}
